@@ -22,24 +22,25 @@
 
 static GHashTable *pixbuf_cache = NULL;
 
-#define CLOCK_FACE_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE ((obj), INTL_TYPE_CLOCK_FACE, ClockFacePrivate))
-
 G_DEFINE_TYPE (ClockFace, clock_face, GTK_TYPE_WIDGET)
 
-static void clock_face_finalize (GObject *);
-static gboolean clock_face_expose (GtkWidget *clock, GdkEventExpose *event);
-static void clock_face_size_request (GtkWidget *clock,
-				     GtkRequisition *requisition);
-static void clock_face_size_allocate (GtkWidget     *clock,
-				      GtkAllocation *allocation);
+static void     clock_face_finalize             (GObject *);
+static gboolean clock_face_draw                 (GtkWidget     *clock,
+                                                 cairo_t       *cr);
+static void     clock_face_get_preferred_width  (GtkWidget     *this,
+                                                 gint          *minimal_width,
+                                                 gint          *natural_width);
+static void     clock_face_get_preferred_height (GtkWidget     *this,
+                                                 gint          *minimal_height,
+                                                 gint          *natural_height);
+static void     clock_face_size_allocate        (GtkWidget     *clock,
+				                 GtkAllocation *allocation);
 
 static void update_time_and_face (ClockFace *this,
                                   gboolean   force_face_loading);
 static void clock_face_load_face (ClockFace *this,
 				  gint width, gint height);
 
-typedef struct _ClockFacePrivate ClockFacePrivate;
-
 typedef enum {
 	CLOCK_FACE_MORNING,
 	CLOCK_FACE_DAY,
@@ -70,8 +71,9 @@ clock_face_class_init (ClockFaceClass *class)
         widget_class = GTK_WIDGET_CLASS (class);
 
         /* GtkWidget signals */
-        widget_class->expose_event = clock_face_expose;
-        widget_class->size_request = clock_face_size_request;
+        widget_class->draw = clock_face_draw;
+        widget_class->get_preferred_width  = clock_face_get_preferred_width;
+        widget_class->get_preferred_height = clock_face_get_preferred_height;
         widget_class->size_allocate = clock_face_size_allocate;
 
         /* GObject signals */
@@ -83,7 +85,9 @@ clock_face_class_init (ClockFaceClass *class)
 static void
 clock_face_init (ClockFace *this)
 {
-        ClockFacePrivate *priv = CLOCK_FACE_GET_PRIVATE (this);
+        ClockFacePrivate *priv;
+
+        priv = this->priv = G_TYPE_INSTANCE_GET_PRIVATE (this, INTL_TYPE_CLOCK_FACE, ClockFacePrivate);
 
         priv->size = CLOCK_FACE_SMALL;
         priv->timeofday = CLOCK_FACE_INVALID;
@@ -93,19 +97,19 @@ clock_face_init (ClockFace *this)
         gtk_widget_set_has_window (GTK_WIDGET (this), FALSE);
 }
 
-static void
-draw (GtkWidget *this, cairo_t *cr)
+static gboolean
+clock_face_draw (GtkWidget *this, cairo_t *cr)
 {
-        ClockFacePrivate *priv;
-        GtkAllocation allocation;
+        ClockFacePrivate *priv = CLOCK_FACE (this)->priv;
+        int width, height;
         double x, y;
         double radius;
         int hours, minutes, seconds;
-
         /* Hand lengths as a multiple of the clock radius */
         double hour_length, min_length, sec_length;
 
-        priv = CLOCK_FACE_GET_PRIVATE (this);
+        if (GTK_WIDGET_CLASS (clock_face_parent_class)->draw)
+                GTK_WIDGET_CLASS (clock_face_parent_class)->draw (this, cr);
 
         if (priv->size == CLOCK_FACE_LARGE) {
                 hour_length = 0.45;
@@ -117,32 +121,21 @@ draw (GtkWidget *this, cairo_t *cr)
                 sec_length = 0.8;   /* not drawn currently */
         }
 
-        gtk_widget_get_allocation (this, &allocation);
+        width = gtk_widget_get_allocated_width (this);
+        height = gtk_widget_get_allocated_width (this);
 
-        x = allocation.x + allocation.width / 2;
-        y = allocation.y + allocation.height / 2;
-        radius = MIN (allocation.width / 2,
-                      allocation.height / 2) - 5;
-
-        cairo_save (cr);
-        cairo_translate (cr, allocation.x, allocation.y);
+        x = width / 2;
+        y = height / 2;
+        radius = MIN (width / 2, height / 2) - 5;
 
         /* clock back */
         if (priv->face_pixbuf) {
-                GdkWindow *window = gtk_widget_get_window (this);
-		gdk_draw_pixbuf (GDK_DRAWABLE (window),
-				 NULL,
-				 priv->face_pixbuf,
-				 0, 0,
-				 allocation.x,
-				 allocation.y,
-				 allocation.width,
-				 allocation.height,
-				 GDK_RGB_DITHER_NONE, 0, 0);
+                cairo_save (cr);
+                gdk_cairo_set_source_pixbuf (cr, priv->face_pixbuf, 0, 0);
+                cairo_paint (cr);
+                cairo_restore (cr);
         }
 
-        cairo_restore (cr);
-
         /* clock hands */
         hours = priv->time.tm_hour;
         minutes = priv->time.tm_min + priv->minute_offset;
@@ -182,24 +175,6 @@ draw (GtkWidget *this, cairo_t *cr)
                 cairo_stroke (cr);
                 cairo_restore (cr);
         }
-}
-
-static gboolean
-clock_face_expose (GtkWidget *this, GdkEventExpose *event)
-{
-        cairo_t *cr;
-
-        /* get a cairo_t */
-        cr = gdk_cairo_create (gtk_widget_get_window (this));
-
-        cairo_rectangle (cr,
-			 event->area.x, event->area.y,
-			 event->area.width, event->area.height);
-        cairo_clip (cr);
-
-        draw (this, cr);
-
-        cairo_destroy (cr);
 
         return FALSE;
 }
@@ -211,40 +186,68 @@ clock_face_redraw_canvas (ClockFace *this)
 }
 
 static void
-clock_face_size_request (GtkWidget *this,
-			 GtkRequisition *requisition)
+clock_face_get_preferred_width (GtkWidget *this,
+                                gint      *minimal_width,
+                                gint      *natural_width)
 {
-        ClockFacePrivate *priv = CLOCK_FACE_GET_PRIVATE (this);
+        ClockFacePrivate *priv = CLOCK_FACE (this)->priv;
 
         if (priv->size_widget != NULL) {
-                GtkRequisition req;
+               int child_minimal_height;
+               int child_natural_height;
 
                 /* Tie our size to the height of the size_widget */
-                gtk_widget_size_request (GTK_WIDGET (priv->size_widget), &req);
+                gtk_widget_get_preferred_height (GTK_WIDGET (priv->size_widget),
+                                                 &child_minimal_height,
+                                                 &child_natural_height);
 
                 /* Pad out our height by a little bit - this improves
                    the balance */
-                requisition->width = req.height + req.height / 8;
-                requisition->height = req.height + req.height / 8;
+                *minimal_width = child_minimal_height + child_minimal_height / 8;
+                *natural_width = child_natural_height + child_natural_height / 8;
         } else if (priv->face_pixbuf != NULL) {
-                int w, h;
-
                 /* Use the size of the current pixbuf */
-                w = gdk_pixbuf_get_width (GDK_PIXBUF (priv->face_pixbuf));
-                h = gdk_pixbuf_get_height (GDK_PIXBUF (priv->face_pixbuf));
-
-                requisition->width = w;
-                requisition->height = h;
+                *minimal_width = *natural_width = gdk_pixbuf_get_width (GDK_PIXBUF (priv->face_pixbuf));
         } else {
                 /* we don't know anything, so use known dimensions for the svg
                  * files */
-                if (priv->size == CLOCK_FACE_LARGE) {
-                        requisition->width = 50;
-                        requisition->height = 50;
-                } else {
-                        requisition->width = 36;
-                        requisition->height = 36;
-                }
+                if (priv->size == CLOCK_FACE_LARGE)
+                        *minimal_width = *natural_width = 50;
+                else
+                        *minimal_width = *natural_width = 36;
+        }
+}
+
+static void
+clock_face_get_preferred_height (GtkWidget *this,
+                                 gint      *minimal_height,
+                                 gint      *natural_height)
+{
+        ClockFacePrivate *priv = CLOCK_FACE (this)->priv;
+
+        if (priv->size_widget != NULL) {
+               int child_minimal_height;
+               int child_natural_height;
+
+                /* Tie our size to the height of the size_widget */
+                gtk_widget_get_preferred_height (GTK_WIDGET (priv->size_widget),
+                                                 &child_minimal_height,
+                                                 &child_natural_height);
+
+                /* Pad out our height by a little bit - this improves
+                   the balance */
+                *minimal_height = child_minimal_height + child_minimal_height / 8;
+                *natural_height = child_natural_height + child_natural_height / 8;
+        } else if (priv->face_pixbuf != NULL) {
+                /* Use the size of the current pixbuf */
+                *minimal_height = *natural_height = gdk_pixbuf_get_height (GDK_PIXBUF (priv->face_pixbuf));
+        } else {
+                /* we don't know anything, so use known dimensions for the svg
+                 * files */
+                if (priv->size == CLOCK_FACE_LARGE)
+                        *minimal_height = *natural_height = 50;
+                else
+                        *minimal_height = *natural_height = 36;
         }
 }
 
@@ -278,7 +281,7 @@ update_time_and_face (ClockFace *this,
         ClockFacePrivate *priv;
 	ClockFaceTimeOfDay timeofday;
 
-        priv = CLOCK_FACE_GET_PRIVATE (this);
+        priv = this->priv;
 
         /* update the time */
         if (priv->location) {
@@ -339,7 +342,7 @@ GtkWidget *
 clock_face_new (ClockFaceSize size)
 {
         GObject *obj = g_object_new (INTL_TYPE_CLOCK_FACE, NULL);
-        ClockFacePrivate *priv = CLOCK_FACE_GET_PRIVATE (obj);
+        ClockFacePrivate *priv = CLOCK_FACE (obj)->priv;
 
         priv->size = size;
 
@@ -352,7 +355,7 @@ clock_face_new_with_location (ClockFaceSize size,
 			      GtkWidget *size_widget)
 {
         GObject *obj = g_object_new (INTL_TYPE_CLOCK_FACE, NULL);
-        ClockFacePrivate *priv = CLOCK_FACE_GET_PRIVATE (obj);
+        ClockFacePrivate *priv = CLOCK_FACE (obj)->priv;
 
         priv->size = size;
         priv->location = g_object_ref (loc);
@@ -364,7 +367,8 @@ clock_face_new_with_location (ClockFaceSize size,
 static void
 clock_face_finalize (GObject *obj)
 {
-        ClockFacePrivate *priv = CLOCK_FACE_GET_PRIVATE (obj);
+        ClockFace *face = CLOCK_FACE (obj);
+        ClockFacePrivate *priv = face->priv;
 
         if (priv->location) {
                 g_object_unref (priv->location);
@@ -400,7 +404,7 @@ remove_pixbuf_from_cache (const char *key,
 static void
 clock_face_load_face (ClockFace *this, gint width, gint height)
 {
-        ClockFacePrivate *priv = CLOCK_FACE_GET_PRIVATE (this);
+        ClockFacePrivate *priv = this->priv;
 	const gchar *size_string[2] = { "small", "large" };
         const gchar *daytime_string[4] = { "morning", "day", "evening", "night" };
 	gchar *cache_name;
