@@ -44,10 +44,39 @@
 #include <cairo-xlib.h>
 
 #include "panel-applet.h"
+#include "panel-applet-bindings.h"
 #include "panel-applet-factory.h"
 #include "panel-applet-marshal.h"
 #include "panel-applet-enums.h"
 
+/**
+ * SECTION:applet
+ * @short_description: a widget embedded in a panel.
+ * @stability: Unstable
+ *
+ * Applets are small applications that are embedded in the GNOME panel. They
+ * can be used to give quick access to some features, or to display the state
+ * of something specific.
+ *
+ * The #PanelApplet API hides all of the embedding process as it handles all
+ * the communication with the GNOME panel. It is a subclass of #GtkBin, so you
+ * can add any kind of widgets to it.
+ *
+ * See the <link linkend="getting-started">Getting Started</link> section to
+ * learn how to properly use #PanelApplet.
+ */
+
+/**
+ * SECTION:applet-factory
+ * @short_description: the factory that will create applets.
+ * @stability: Unstable
+ *
+ * This API is used to create an <link
+ * linkend="getting-started.concepts.applet-factory">applet factory</link>. You
+ * need to call one and only one of these functions to get applets working in
+ * your binary.
+ */
+
 #define PANEL_APPLET_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_APPLET, PanelAppletPrivate))
 
 struct _PanelAppletPrivate {
@@ -60,6 +89,7 @@ struct _PanelAppletPrivate {
 	GClosure          *closure;
 	char              *object_path;
 	guint              object_id;
+	char              *settings_path;
 	char              *prefs_key;
 
 	GtkUIManager      *ui_manager;
@@ -80,7 +110,6 @@ struct _PanelAppletPrivate {
 
 	gboolean           moving_focus_out;
 
-	gboolean           locked;
 	gboolean           locked_down;
 };
 
@@ -89,7 +118,6 @@ enum {
         CHANGE_SIZE,
         CHANGE_BACKGROUND,
 	MOVE_FOCUS_OUT_OF_APPLET,
-        SAVE_YOURSELF,
         LAST_SIGNAL
 };
 
@@ -100,13 +128,13 @@ enum {
 	PROP_ID,
 	PROP_CLOSURE,
 	PROP_CONNECTION,
+	PROP_SETTINGS_PATH,
 	PROP_PREFS_KEY,
 	PROP_ORIENT,
 	PROP_SIZE,
 	PROP_BACKGROUND,
 	PROP_FLAGS,
 	PROP_SIZE_HINTS,
-	PROP_LOCKED,
 	PROP_LOCKED_DOWN
 };
 
@@ -118,23 +146,23 @@ static void       panel_applet_menu_cmd_remove     (GtkAction         *action,
 						    PanelApplet       *applet);
 static void       panel_applet_menu_cmd_move       (GtkAction         *action,
 						    PanelApplet       *applet);
-static void       panel_applet_menu_cmd_lock       (GtkAction         *action,
-						    PanelApplet       *applet);
 static void       panel_applet_register_object     (PanelApplet       *applet);
 
 static const gchar panel_menu_ui[] =
 	"<ui>\n"
 	"  <popup name=\"PanelAppletPopup\" action=\"PopupAction\">\n"
 	"    <placeholder name=\"AppletItems\"/>\n"
-	"    <separator/>\n"
-	"    <menuitem name=\"RemoveItem\" action=\"Remove\"/>\n"
+	"  </popup>\n"
+	"  <popup name=\"PanelAppletEditPopup\" action=\"PopupEditAction\">\n"
 	"    <menuitem name=\"MoveItem\" action=\"Move\"/>\n"
-	"    <separator/>\n"
-	"    <menuitem name=\"LockItem\" action=\"Lock\"/>\n"
+	"    <menuitem name=\"RemoveItem\" action=\"Remove\"/>\n"
 	"  </popup>\n"
 	"</ui>\n";
 
 static const GtkActionEntry menu_entries[] = {
+	{ "PopupAction", NULL, "Popup Action",
+	  NULL, NULL,
+	  NULL },
 	{ "Remove", GTK_STOCK_REMOVE, N_("_Remove From Panel"),
 	  NULL, NULL,
 	  G_CALLBACK (panel_applet_menu_cmd_remove) },
@@ -143,12 +171,6 @@ static const GtkActionEntry menu_entries[] = {
 	  G_CALLBACK (panel_applet_menu_cmd_move) }
 };
 
-static const GtkToggleActionEntry menu_toggle_entries[] = {
-	{ "Lock", NULL, N_("Loc_k To Panel"),
-	  NULL, NULL,
-	  G_CALLBACK (panel_applet_menu_cmd_lock) }
-};
-
 G_DEFINE_TYPE (PanelApplet, panel_applet, GTK_TYPE_EVENT_BOX)
 
 #define PANEL_APPLET_INTERFACE   "org.gnome.panel.applet.Applet"
@@ -162,6 +184,8 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 {
 	GSList *list, *l;
 
+	g_assert (error != NULL);
+
 	list = gconf_client_all_entries (client, schema_dir, error);
 
 	if (*error != NULL)
@@ -256,12 +280,25 @@ panel_applet_associate_schemas_in_dir (GConfClient  *client,
 	g_slist_free (list);
 }
 
+/**
+ * panel_applet_add_preferences:
+ * @applet: a #PanelApplet.
+ * @schema_dir: a GConf path of a schema directory.
+ * @error: a #GError, or %NULL.
+ *
+ * Associates the per-instance GConf directory of @applet to the schemas
+ * defined in @schema_dir. This must be called if the applet will save
+ * per-instance settings, to ensure that each key in the per-instance GConf
+ * directory has a defined type, sane default and documentation.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 void
 panel_applet_add_preferences (PanelApplet  *applet,
 			      const gchar  *schema_dir,
-			      GError      **opt_error)
+			      GError      **error)
 {
-	GError **error = NULL;
+	GError **_error = NULL;
 	GError  *our_error = NULL;
 
 	g_return_if_fail (PANEL_IS_APPLET (applet));
@@ -270,23 +307,31 @@ panel_applet_add_preferences (PanelApplet  *applet,
 	if (!applet->priv->prefs_key)
 		return;
 
-	if (opt_error)
-		error = opt_error;
+	/* panel_applet_associate_schemas_in_dir() requires a non-NULL error */
+	if (error)
+		_error = error;
 	else
-		error = &our_error;
+		_error = &our_error;
 
 	panel_applet_associate_schemas_in_dir (applet->priv->client,
 					       applet->priv->prefs_key,
 					       schema_dir,
-					       error);
+					       _error);
 
-	if (!opt_error && our_error) {
-		g_warning (G_STRLOC ": failed to add preferences from '%s' : '%s'",
-			   schema_dir, our_error->message);
+	if (!error && our_error)
 		g_error_free (our_error);
-	}
 }
 
+/**
+ * panel_applet_get_preferences_key:
+ * @applet: a #PanelApplet.
+ *
+ * Gets the GConf path to the per-instance settings of @applet.
+ *
+ * Returns: a copy of the GConf path to the per-instance settings of @applet.
+ *
+ * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+ **/
 char *
 panel_applet_get_preferences_key (PanelApplet *applet)
 {
@@ -329,6 +374,56 @@ panel_applet_set_preferences_key (PanelApplet *applet,
 	g_object_notify (G_OBJECT (applet), "prefs-key");
 }
 
+/**
+ * panel_applet_settings_new:
+ * @applet: a #PanelApplet.
+ * @schema: the name of the schema.
+ *
+ * Creates a new #GSettings object for the per-instance settings of @applet,
+ * with a given schema.
+ *
+ * Returns: a new #GSettings object for the per-instance settings of @applet.
+ **/
+GSettings *
+panel_applet_settings_new (PanelApplet *applet,
+			   const char  *schema)
+{
+	g_return_val_if_fail (PANEL_IS_APPLET (applet), NULL);
+	g_return_val_if_fail (schema != NULL, NULL);
+
+	if (!applet->priv->settings_path)
+		return NULL;
+
+	return g_settings_new_with_path (schema, applet->priv->settings_path);
+}
+
+static void
+panel_applet_set_settings_path (PanelApplet *applet,
+				const char  *settings_path)
+{
+	if (applet->priv->settings_path == settings_path)
+		return;
+
+	if (g_strcmp0 (applet->priv->settings_path, settings_path) == 0)
+		return;
+
+	g_free (applet->priv->settings_path);
+	applet->priv->settings_path = NULL;
+
+	if (settings_path)
+		applet->priv->settings_path = g_strdup (settings_path);
+
+	g_object_notify (G_OBJECT (applet), "settings-path");
+}
+
+/**
+ * panel_applet_get_flags:
+ * @applet: a #PanelApplet.
+ *
+ * Gets the #PanelAppletFlags of @applet.
+ *
+ * Returns: the #PanelAppletFlags of @applet.
+ **/
 PanelAppletFlags
 panel_applet_get_flags (PanelApplet *applet)
 {
@@ -337,6 +432,14 @@ panel_applet_get_flags (PanelApplet *applet)
 	return applet->priv->flags;
 }
 
+/**
+ * panel_applet_set_flags:
+ * @applet: a #PanelApplet.
+ * @flags: #PanelAppletFlags to use for @applet.
+ *
+ * Sets the #PanelAppletFlags of @applet. Most of the time, at least
+ * %PANEL_APPLET_EXPAND_MINOR should be used.
+ **/
 void
 panel_applet_set_flags (PanelApplet      *applet,
 			PanelAppletFlags  flags)
@@ -414,6 +517,28 @@ panel_applet_size_hints_changed (PanelApplet *applet,
 	return FALSE;
 }
 
+/**
+ * panel_applet_set_size_hints:
+ * @applet: a #PanelApplet.
+ * @size_hints: array of sizes.
+ * @n_elements: length of @size_hints.
+ * @base_size: base size of the applet.
+ *
+ * Give hints to the panel about sizes @applet is comfortable with. This is
+ * generally useful for applets that can take a lot of space, in case the panel
+ * gets full and needs to restrict the size of some applets.
+ *
+ * @size_hints should have an even number of sizes. It is an array of (max,
+ * min) pairs where min(i) > max(i + 1).
+ *
+ * @base_size will be added to all sizes in @size_hints, and is therefore a way
+ * to guarantee a minimum size to @applet.
+ *
+ * The panel will try to allocate a size that is acceptable to @applet, i.e. in
+ * one of the (@base_size + max, @base_size + min) ranges.
+ *
+ * %PANEL_APPLET_EXPAND_MAJOR must be set for @applet to use size hints.
+ **/
 void
 panel_applet_set_size_hints (PanelApplet *applet,
 			     const int   *size_hints,
@@ -467,6 +592,18 @@ panel_applet_set_size_hints (PanelApplet *applet,
 	}
 }
 
+/**
+ * panel_applet_get_size:
+ * @applet: a #PanelApplet.
+ *
+ * Gets the size of the panel @applet is on. For a horizontal panel, the
+ * size if the height of the panel; for a vertical panel, the size is the width
+ * of the panel.
+ *
+ * Returns: the size of the panel @applet is on.
+ *
+ * Deprecated: 3.0: Use the allocation of @applet instead.
+ **/
 guint
 panel_applet_get_size (PanelApplet *applet)
 {
@@ -493,6 +630,14 @@ panel_applet_set_size (PanelApplet *applet,
 	g_object_notify (G_OBJECT (applet), "size");
 }
 
+/**
+ * panel_applet_get_orient:
+ * @applet: a #PanelApplet.
+ *
+ * Gets the #PanelAppletOrient of @applet.
+ *
+ * Returns: the #PanelAppletOrient of @applet.
+ **/
 PanelAppletOrient
 panel_applet_get_orient (PanelApplet *applet)
 {
@@ -519,62 +664,15 @@ panel_applet_set_orient (PanelApplet      *applet,
 	g_object_notify (G_OBJECT (applet), "orient");
 }
 
-#if 0
-/* Locked should not be public API: it's not useful for applet writers to know
- * if the applet is locked (as opposed to locked_down). */
-static gboolean
-panel_applet_get_locked (PanelApplet *applet)
-{
-	g_return_val_if_fail (PANEL_IS_APPLET (applet), FALSE);
-
-	return applet->priv->locked;
-}
-#endif
-
-static void
-panel_applet_set_locked (PanelApplet *applet,
-			 gboolean     locked)
-{
-	GtkAction *action;
-
-	g_return_if_fail (PANEL_IS_APPLET (applet));
-
-	if (applet->priv->locked == locked)
-		return;
-
-	applet->priv->locked = locked;
-
-	action = panel_applet_menu_get_action (applet, "Lock");
-	g_signal_handlers_block_by_func (action,
-					 panel_applet_menu_cmd_lock,
-					 applet);
-	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action), locked);
-	g_signal_handlers_unblock_by_func (action,
-					   panel_applet_menu_cmd_lock,
-					   applet);
-
-	panel_applet_menu_update_actions (applet);
-
-	g_object_notify (G_OBJECT (applet), "locked");
-
-	if (applet->priv->connection) {
-		GError *error = NULL;
-
-		g_dbus_connection_emit_signal (applet->priv->connection,
-					       NULL,
-					       applet->priv->object_path,
-					       PANEL_APPLET_INTERFACE,
-					       locked ? "Lock" : "Unlock",
-					       NULL, &error);
-		if (error) {
-			g_printerr ("Failed to send signal %s: %s\n",
-				    locked ? "Lock" : "Unlock",
-				    error->message);
-			g_error_free (error);
-		}
-	}
-}
-
+/**
+ * panel_applet_get_locked_down:
+ * @applet: a #PanelApplet.
+ *
+ * Gets whether the panel @applet is on is locked down or not. A locked down
+ * applet should not allow any change to its configuration.
+ *
+ * Returns: %TRUE if the panel @applet is on is locked down, %FALSE otherwise.
+ **/
 gboolean
 panel_applet_get_locked_down (PanelApplet *applet)
 {
@@ -683,6 +781,15 @@ panel_applet_find_toplevel_dock_window (PanelApplet *applet,
 	return None;
 }
 
+/**
+ * panel_applet_request_focus:
+ * @applet: a #PanelApplet.
+ * @timestamp: the timestamp of the user interaction (typically a button or key
+ * press event) which triggered this call.
+ *
+ * Requests focus for @applet. There is no guarantee that @applet will
+ * successfully get focus after that call.
+ **/
 /* This function
  *   1) Gets the window id of the panel that contains the applet
  *	using XQueryTree and XGetWindowProperty to find an ancestor
@@ -744,17 +851,12 @@ panel_applet_menu_get_action (PanelApplet *applet,
 static void
 panel_applet_menu_update_actions (PanelApplet *applet)
 {
-	gboolean locked = applet->priv->locked;
 	gboolean locked_down = applet->priv->locked_down;
 
-	g_object_set (panel_applet_menu_get_action (applet, "Lock"),
-		      "visible", !locked_down, NULL);
 	g_object_set (panel_applet_menu_get_action (applet, "Move"),
-		      "sensitive", !locked,
 		      "visible", !locked_down,
 		      NULL);
 	g_object_set (panel_applet_menu_get_action (applet, "Remove"),
-		      "sensitive", !locked,
 		      "visible", !locked_down,
 		      NULL);
 }
@@ -803,20 +905,23 @@ panel_applet_menu_cmd_move (GtkAction   *action,
 	}
 }
 
-static void
-panel_applet_menu_cmd_lock (GtkAction   *action,
-			    PanelApplet *applet)
-{
-	gboolean locked;
-
-	locked = gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action));
-	panel_applet_set_locked (applet, locked);
-}
-
+/**
+ * panel_applet_setup_menu:
+ * @applet: a #PanelApplet.
+ * @xml: a menu XML string.
+ * @action_group: a #GtkActionGroup.
+ *
+ * Sets up the context menu of @applet. @xml is a #GtkUIManager UI definition,
+ * describing how to display the menu items. @action_group contains the
+ * various #GtkAction that are referenced in @xml.
+ *
+ * See also the <link linkend="getting-started.context-menu">Context
+ * Menu</link> section.
+ **/
 void
 panel_applet_setup_menu (PanelApplet    *applet,
 			 const gchar    *xml,
-			 GtkActionGroup *applet_action_group)
+			 GtkActionGroup *action_group)
 {
 	gchar  *new_xml;
 	GError *error = NULL;
@@ -827,9 +932,9 @@ panel_applet_setup_menu (PanelApplet    *applet,
 	if (applet->priv->applet_action_group)
 		return;
 
-	applet->priv->applet_action_group = g_object_ref (applet_action_group);
+	applet->priv->applet_action_group = g_object_ref (action_group);
 	gtk_ui_manager_insert_action_group (applet->priv->ui_manager,
-					    applet_action_group, 0);
+					    action_group, 0);
 
 	new_xml = g_strdup_printf ("<ui><popup name=\"PanelAppletPopup\" action=\"AppletItems\">"
 				   "<placeholder name=\"AppletItems\">%s\n</placeholder>\n"
@@ -843,16 +948,30 @@ panel_applet_setup_menu (PanelApplet    *applet,
 	}
 }
 
+/**
+ * panel_applet_setup_menu_from_file:
+ * @applet: a #PanelApplet.
+ * @filename: path to a menu XML file.
+ * @action_group: a #GtkActionGroup.
+ *
+ * Sets up the context menu of @applet. @filename is the path to a menu XML
+ * file, containing a #GtkUIManager UI definition that describes how to display
+ * the menu items. @action_group contains the various #GtkAction that
+ * are referenced in @xml.
+ *
+ * See also the <link linkend="getting-started.context-menu">Context
+ * Menu</link> section.
+ **/
 void
 panel_applet_setup_menu_from_file (PanelApplet    *applet,
 				   const gchar    *filename,
-				   GtkActionGroup *applet_action_group)
+				   GtkActionGroup *action_group)
 {
 	gchar  *xml = NULL;
 	GError *error = NULL;
 
 	if (g_file_get_contents (filename, &xml, NULL, &error)) {
-		panel_applet_setup_menu (applet, xml, applet_action_group);
+		panel_applet_setup_menu (applet, xml, action_group);
 	} else {
 		g_warning ("%s", error->message);
 		g_error_free (error);
@@ -882,6 +1001,8 @@ panel_applet_finalize (GObject *object)
 
 	panel_applet_set_preferences_key (applet, NULL);
 
+	panel_applet_bindings_clean (applet->priv->client);
+
 	if (applet->priv->client)
 		g_object_unref (applet->priv->client);
 	applet->priv->client = NULL;
@@ -903,6 +1024,7 @@ panel_applet_finalize (GObject *object)
 
 	g_free (applet->priv->size_hints);
 	g_free (applet->priv->prefs_key);
+	g_free (applet->priv->settings_path);
 	g_free (applet->priv->background);
 	g_free (applet->priv->id);
 
@@ -1014,9 +1136,23 @@ panel_applet_menu_popup (PanelApplet *applet,
 			 guint32      time)
 {
 	GtkWidget *menu;
+	GList     *children, *l;
+	gboolean   visible = FALSE;
 
 	menu = gtk_ui_manager_get_widget (applet->priv->ui_manager,
 					  "/PanelAppletPopup");
+
+	children = gtk_container_get_children (GTK_CONTAINER (menu));
+	for (l = children; l != NULL; l = l->next) {
+		visible = gtk_widget_get_visible (GTK_WIDGET (l->data));
+		if (visible)
+			break;
+	}
+	g_list_free (children);
+
+	if (!visible)
+		return;
+
 	gtk_menu_popup (GTK_MENU (menu),
 			NULL, NULL,
 			(GtkMenuPositionFunc) panel_applet_position_menu,
@@ -1024,6 +1160,25 @@ panel_applet_menu_popup (PanelApplet *applet,
 			button, time);
 }
 
+static void
+panel_applet_edit_menu_popup (PanelApplet *applet,
+			      guint        button,
+			      guint32      time)
+{
+	GtkWidget *menu;
+
+	if (applet->priv->locked_down)
+		return;
+
+	menu = gtk_ui_manager_get_widget (applet->priv->ui_manager,
+					  "/PanelAppletEditPopup");
+	gtk_menu_popup (GTK_MENU (menu),
+			NULL, NULL,
+			NULL,
+			applet,
+			button, time);
+}
+
 static gboolean
 panel_applet_can_focus (GtkWidget *widget)
 {
@@ -1109,7 +1264,14 @@ panel_applet_button_press (GtkWidget      *widget,
 	}
 
 	if (event->button == 3) {
-		panel_applet_menu_popup (applet, event->button, event->time);
+		guint modifiers;
+
+		modifiers = event->state & GDK_MODIFIER_MASK;
+
+		if (modifiers == panel_applet_bindings_get_mouse_button_modifier_keymask ())
+			panel_applet_edit_menu_popup (applet, event->button, event->time);
+		else
+			panel_applet_menu_popup (applet, event->button, event->time);
 
 		return TRUE;
 	}
@@ -1127,11 +1289,20 @@ panel_applet_button_release (GtkWidget      *widget,
 }
 
 static gboolean
-panel_applet_popup_menu (GtkWidget *widget)
+panel_applet_key_press_event (GtkWidget   *widget,
+			      GdkEventKey *event)
 {
-	panel_applet_menu_popup (PANEL_APPLET (widget), 3, GDK_CURRENT_TIME);
+	gboolean is_popup = FALSE;
+	gboolean is_edit_popup = FALSE;
 
-	return TRUE;
+	panel_applet_bindings_key_event_is_popup (event, &is_popup, &is_edit_popup);
+
+	if (is_edit_popup)
+		panel_applet_edit_menu_popup (PANEL_APPLET (widget), 3, event->time);
+	else if (is_popup)
+		panel_applet_menu_popup (PANEL_APPLET (widget), 3, event->time);
+
+	return (is_popup || is_edit_popup);
 }
 
 static GtkSizeRequestMode
@@ -1433,12 +1604,13 @@ panel_applet_get_pattern_from_pixmap (PanelApplet *applet,
 
 /**
  * panel_applet_get_background:
- * @applet: a #PanelApplet
+ * @applet: a #PanelApplet.
  *
- * Returns the background pattern for @applet, or %NULL if there is none.
+ * Gets the background pattern for @applet, or %NULL if there is none.
  *
- * Returns: (transfer full): a new #cairo_pattern_t
- */
+ * Returns: (transfer full): a new #cairo_pattern_t to use as background for
+ * @applet.
+ **/
 cairo_pattern_t *
 panel_applet_get_background (PanelApplet *applet)
 {
@@ -1643,6 +1815,9 @@ panel_applet_get_property (GObject    *object,
 	case PROP_CONNECTION:
 		g_value_set_object (value, applet->priv->connection);
 		break;
+	case PROP_SETTINGS_PATH:
+		g_value_set_string (value, applet->priv->settings_path);
+		break;
 	case PROP_PREFS_KEY:
 		g_value_set_string (value, applet->priv->prefs_key);
 		break;
@@ -1672,9 +1847,6 @@ panel_applet_get_property (GObject    *object,
 		g_value_set_pointer (value, variant);
 	}
 		break;
-	case PROP_LOCKED:
-		g_value_set_boolean (value, applet->priv->locked);
-		break;
 	case PROP_LOCKED_DOWN:
 		g_value_set_boolean (value, applet->priv->locked_down);
 		break;
@@ -1697,12 +1869,19 @@ panel_applet_set_property (GObject      *object,
 		break;
 	case PROP_CLOSURE:
 		applet->priv->closure = g_value_get_pointer (value);
-		g_closure_set_marshal (applet->priv->closure,
-				       panel_applet_marshal_BOOLEAN__STRING);
+		/* We know closure should not be NULL, but we'll fail in
+		 * panel_applet_constructed() with a proper message if this is
+		 * the case. */
+		if (applet->priv->closure)
+			g_closure_set_marshal (applet->priv->closure,
+					       panel_applet_marshal_BOOLEAN__STRING);
 		break;
 	case PROP_CONNECTION:
 		applet->priv->connection = g_value_dup_object (value);
 		break;
+	case PROP_SETTINGS_PATH:
+		panel_applet_set_settings_path (applet, g_value_get_string (value));
+		break;
 	case PROP_PREFS_KEY:
 		panel_applet_set_preferences_key (applet, g_value_get_string (value));
 		break;
@@ -1727,9 +1906,6 @@ panel_applet_set_property (GObject      *object,
 		panel_applet_set_size_hints (applet, size_hints, n_elements, 0);
 	}
 		break;
-	case PROP_LOCKED:
-		panel_applet_set_locked (applet, g_value_get_boolean (value));
-		break;
 	case PROP_LOCKED_DOWN:
 		panel_applet_set_locked_down (applet, g_value_get_boolean (value));
 		break;
@@ -1812,10 +1988,6 @@ panel_applet_init (PanelApplet *applet)
 				      menu_entries,
 				      G_N_ELEMENTS (menu_entries),
 				      applet);
-	gtk_action_group_add_toggle_actions (applet->priv->panel_action_group,
-					     menu_toggle_entries,
-					     G_N_ELEMENTS (menu_toggle_entries),
-					     applet);
 
 	applet->priv->ui_manager = gtk_ui_manager_new ();
 	gtk_ui_manager_insert_action_group (applet->priv->ui_manager,
@@ -1824,6 +1996,7 @@ panel_applet_init (PanelApplet *applet)
 					   panel_menu_ui, -1, NULL);
 
 
+	panel_applet_bindings_init (applet->priv->client);
 
 
 	applet->priv->plug = gtk_plug_new (0);
@@ -1843,7 +2016,12 @@ panel_applet_constructed (GObject *object)
 {
 	PanelApplet *applet = PANEL_APPLET (object);
 
-	panel_applet_register_object (applet);
+	if (!applet->priv->connection || !applet->priv->closure || !applet->priv->id) {
+		g_printerr ("Bad use of PanelApplet API: you should not create a PanelApplet object yourself. Please use panel_applet_factory_main() instead.\n");
+		g_assert_not_reached ();
+	}
+
+	return panel_applet_register_object (applet);
 }
 
 static void
@@ -1862,6 +2040,7 @@ panel_applet_class_init (PanelAppletClass *klass)
 
 	widget_class->button_press_event = panel_applet_button_press;
 	widget_class->button_release_event = panel_applet_button_release;
+	widget_class->key_press_event = panel_applet_key_press_event;
 	widget_class->get_request_mode = panel_applet_get_request_mode;
         widget_class->get_preferred_width = panel_applet_get_preferred_width;
         widget_class->get_preferred_height = panel_applet_get_preferred_height;
@@ -1869,10 +2048,14 @@ panel_applet_class_init (PanelAppletClass *klass)
 	widget_class->draw = panel_applet_draw;
 	widget_class->focus = panel_applet_focus;
 	widget_class->realize = panel_applet_realize;
-	widget_class->popup_menu = panel_applet_popup_menu;
 
 	g_type_class_add_private (klass, sizeof (PanelAppletPrivate));
 
+	/**
+	 * PanelApplet:id: (skip)
+	 *
+	 * Implementation detail.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_ID,
 					 g_param_spec_string ("id",
@@ -1881,6 +2064,11 @@ panel_applet_class_init (PanelAppletClass *klass)
 							      NULL,
 							      G_PARAM_CONSTRUCT_ONLY |
 							      G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:closure: (skip)
+	 *
+	 * Implementation detail.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_CLOSURE,
 					 g_param_spec_pointer ("closure",
@@ -1888,6 +2076,11 @@ panel_applet_class_init (PanelAppletClass *klass)
 							       "The Applet closure",
 							       G_PARAM_CONSTRUCT_ONLY |
 							       G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:connection: (skip)
+	 *
+	 * Implementation detail.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_CONNECTION,
 					 g_param_spec_object ("connection",
@@ -1896,6 +2089,29 @@ panel_applet_class_init (PanelAppletClass *klass)
 							      G_TYPE_DBUS_CONNECTION,
 							      G_PARAM_CONSTRUCT_ONLY |
 							      G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:settings-path:
+	 *
+	 * The GSettings path to the per-instance settings of the applet.
+	 *
+	 * This property gets set when the applet gets embedded.
+	 **/
+	g_object_class_install_property (gobject_class,
+					 PROP_SETTINGS_PATH,
+					 g_param_spec_string ("settings-path",
+							      "SettingsPath",
+							      "GSettings path to per-instance settings",
+							      NULL,
+							      G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:prefs-key:
+	 *
+	 * The GConf path to the per-instance settings of the applet.
+	 *
+	 * This property gets set when the applet gets embedded.
+	 *
+	 * Deprecated: 3.0: Use #GSettings to store per-instance settings.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_PREFS_KEY,
 					 g_param_spec_string ("prefs-key",
@@ -1903,14 +2119,35 @@ panel_applet_class_init (PanelAppletClass *klass)
 							      "GConf Preferences Key",
 							      NULL,
 							      G_PARAM_READWRITE));
-        // FIXMEchpe GtkOrientation?
+	/**
+	 * PanelApplet:orient:
+	 *
+	 * The #PanelAppletOrient of the applet.
+	 *
+	 * This property gets set when the applet gets embedded, and can change
+	 * when the panel position changes.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_ORIENT,
 					 g_param_spec_uint ("orient",
 							    "Orient",
 							    "Panel Applet Orientation",
-							    0, G_MAXUINT, 0, /* FIXME */
+							    PANEL_APPLET_ORIENT_FIRST,
+							    PANEL_APPLET_ORIENT_LAST,
+							    PANEL_APPLET_ORIENT_UP,
 							    G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:size:
+	 *
+	 * The size of the panel the applet is on. For a horizontal panel, the
+	 * size if the height of the panel; for a vertical panel, the size is
+	 * the width of the panel.
+	 *
+	 * This property gets set when the applet gets embedded, and can change
+	 * when the panel size changes.
+         *
+	 * Deprecated: 3.0: Use the allocation of @applet instead.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_SIZE,
 					 g_param_spec_uint ("size",
@@ -1918,6 +2155,11 @@ panel_applet_class_init (PanelAppletClass *klass)
 							    "Panel Applet Size",
 							    0, G_MAXUINT, 0,
 							    G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:background: (skip)
+	 *
+	 * Implementation detail.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_BACKGROUND,
 					 g_param_spec_string ("background",
@@ -1925,35 +2167,52 @@ panel_applet_class_init (PanelAppletClass *klass)
 							      "Panel Applet Background",
 							      NULL,
 							      G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:flags:
+	 *
+	 * The #PanelAppletFlags of the applet.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_FLAGS,
 					 g_param_spec_uint ("flags",
 							    "Flags",
 							    "Panel Applet flags",
-							    0, G_MAXUINT, 0, /* FIXME */
+							    PANEL_APPLET_FLAGS_NONE,
+							    PANEL_APPLET_FLAGS_ALL,
+							    PANEL_APPLET_FLAGS_NONE,
 							    G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:size-hints:
+	 *
+	 * The size hints set for the applet. See panel_applet_set_size_hints().
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_SIZE_HINTS,
 					 /* FIXME: value_array? */
 					 g_param_spec_pointer ("size-hints",
 							       "SizeHints",
-							       "Panel Applet Size Hints",
-							       G_PARAM_READWRITE));
-	g_object_class_install_property (gobject_class,
-					 PROP_LOCKED,
-					 g_param_spec_boolean ("locked",
-							       "Locked",
-							       "Whether Panel Applet is locked",
-							       FALSE,
+							       "Size hints of the applet",
 							       G_PARAM_READWRITE));
+	/**
+	 * PanelApplet:locked-down:
+	 *
+	 * Whether the panel the applet is on is locked down.
+	 **/
 	g_object_class_install_property (gobject_class,
 					 PROP_LOCKED_DOWN,
 					 g_param_spec_boolean ("locked-down",
 							       "LockedDown",
-							       "Whether Panel Applet is locked down",
+							       "Whether the panel the applet is on is locked down",
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+        /**
+         * PanelApplet::change-orient:
+         * @applet: the #PanelApplet which emitted the signal.
+         * @orient: the new #PanelAppletOrient of @applet.
+         *
+         * Emitted when the #PanelAppletOrient of @applet has changed.
+         **/
 	panel_applet_signals [CHANGE_ORIENT] =
                 g_signal_new ("change_orient",
                               G_TYPE_FROM_CLASS (klass),
@@ -1966,6 +2225,15 @@ panel_applet_class_init (PanelAppletClass *klass)
 			      1,
 			      G_TYPE_UINT);
 
+        /**
+         * PanelApplet::change-size:
+         * @applet: the #PanelApplet which emitted the signal.
+         * @size: the new size of the panel @applet is on.
+         *
+         * Emitted when the size of the panel @applet is on has changed.
+	 *
+	 * Deprecated: 3.0: Use the #GtkWidget::size-allocate signal instead.
+         **/
 	panel_applet_signals [CHANGE_SIZE] =
                 g_signal_new ("change_size",
                               G_TYPE_FROM_CLASS (klass),
@@ -1978,6 +2246,13 @@ panel_applet_class_init (PanelAppletClass *klass)
 			      1,
 			      G_TYPE_INT);
 
+        /**
+         * PanelApplet::change-background:
+         * @applet: the #PanelApplet which emitted the signal.
+         * @pattern: the new background pattern for @applet, or %NULL if there is none.
+         *
+         * Emitted when the background of @applet has changed.
+         **/
 	panel_applet_signals [CHANGE_BACKGROUND] =
                 g_signal_new ("change_background",
                               G_TYPE_FROM_CLASS (klass),
@@ -1990,6 +2265,14 @@ panel_applet_class_init (PanelAppletClass *klass)
 			      1,
 			      CAIRO_GOBJECT_TYPE_PATTERN);
 
+        /**
+         * PanelApplet::move-focus-out-of-applet: (skip)
+         * @applet: the #PanelApplet which emitted the signal.
+         * @direction: the move direction.
+         *
+	 * Emitted when the focus is moved out of @applet. This is an
+	 * implementation detail.
+         **/
 	panel_applet_signals [MOVE_FOCUS_OUT_OF_APPLET] =
                 g_signal_new ("move_focus_out_of_applet",
                               G_TYPE_FROM_CLASS (klass),
@@ -2009,16 +2292,6 @@ panel_applet_class_init (PanelAppletClass *klass)
 	add_tab_bindings (binding_set, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
 }
 
-GtkWidget *
-panel_applet_new (void)
-{
-	PanelApplet *applet;
-
-	applet = g_object_new (PANEL_TYPE_APPLET, NULL);
-
-	return GTK_WIDGET (applet);
-}
-
 static void
 method_call_cb (GDBusConnection       *connection,
 		const gchar           *sender,
@@ -2038,6 +2311,14 @@ method_call_cb (GDBusConnection       *connection,
 		g_variant_get (parameters, "(uu)", &button, &time);
 		panel_applet_menu_popup (applet, button, time);
 
+		g_dbus_method_invocation_return_value (invocation, NULL);
+	} else if (g_strcmp0 (method_name, "PopupEditMenu") == 0) {
+		guint button;
+		guint time;
+
+		g_variant_get (parameters, "(uu)", &button, &time);
+		panel_applet_edit_menu_popup (applet, button, time);
+
 		g_dbus_method_invocation_return_value (invocation, NULL);
 	}
 }
@@ -2054,7 +2335,10 @@ get_property_cb (GDBusConnection *connection,
 	PanelApplet *applet = PANEL_APPLET (user_data);
 	GVariant    *retval = NULL;
 
-	if (g_strcmp0 (property_name, "PrefsKey") == 0) {
+	if (g_strcmp0 (property_name, "SettingsPath") == 0) {
+		retval = g_variant_new_string (applet->priv->settings_path ?
+					       applet->priv->settings_path : "");
+	} else if (g_strcmp0 (property_name, "PrefsKey") == 0) {
 		retval = g_variant_new_string (applet->priv->prefs_key ?
 					       applet->priv->prefs_key : "");
 	} else if (g_strcmp0 (property_name, "Orient") == 0) {
@@ -2076,8 +2360,6 @@ get_property_cb (GDBusConnection *connection,
 		retval = g_variant_new_array (G_VARIANT_TYPE_INT32,
 					      children, applet->priv->size_hints_len);
 		g_free (children);
-	} else if (g_strcmp0 (property_name, "Locked") == 0) {
-		retval = g_variant_new_boolean (applet->priv->locked);
 	} else if (g_strcmp0 (property_name, "LockedDown") == 0) {
 		retval = g_variant_new_boolean (applet->priv->locked_down);
 	}
@@ -2097,7 +2379,9 @@ set_property_cb (GDBusConnection *connection,
 {
 	PanelApplet *applet = PANEL_APPLET (user_data);
 
-	if (g_strcmp0 (property_name, "PrefsKey") == 0) {
+	if (g_strcmp0 (property_name, "SettingsPath") == 0) {
+		panel_applet_set_settings_path (applet, g_variant_get_string (value, NULL));
+	} else if (g_strcmp0 (property_name, "PrefsKey") == 0) {
 		panel_applet_set_preferences_key (applet, g_variant_get_string (value, NULL));
 	} else if (g_strcmp0 (property_name, "Orient") == 0) {
 		panel_applet_set_orient (applet, g_variant_get_uint32 (value));
@@ -2113,8 +2397,6 @@ set_property_cb (GDBusConnection *connection,
 
 		size_hints = g_variant_get_fixed_array (value, &n_elements, sizeof (gint32));
 		panel_applet_set_size_hints (applet, size_hints, n_elements, 0);
-	} else if (g_strcmp0 (property_name, "Locked") == 0) {
-		panel_applet_set_locked (applet, g_variant_get_boolean (value));
 	} else if (g_strcmp0 (property_name, "LockedDown") == 0) {
 		panel_applet_set_locked_down (applet, g_variant_get_boolean (value));
 	}
@@ -2129,18 +2411,20 @@ static const gchar introspection_xml[] =
 	      "<arg name='button' type='u' direction='in'/>"
 	      "<arg name='time' type='u' direction='in'/>"
 	    "</method>"
+	    "<method name='PopupEditMenu'>"
+	      "<arg name='button' type='u' direction='in'/>"
+	      "<arg name='time' type='u' direction='in'/>"
+	    "</method>"
+	    "<property name='SettingsPath' type='s' access='readwrite'/>"
 	    "<property name='PrefsKey' type='s' access='readwrite'/>"
 	    "<property name='Orient' type='u' access='readwrite' />"
 	    "<property name='Size' type='u' access='readwrite'/>"
 	    "<property name='Background' type='s' access='readwrite'/>"
 	    "<property name='Flags' type='u' access='readwrite'/>"
 	    "<property name='SizeHints' type='ai' access='readwrite'/>"
-	    "<property name='Locked' type='b' access='readwrite'/>"
 	    "<property name='LockedDown' type='b' access='readwrite'/>"
 	    "<signal name='Move' />"
 	    "<signal name='RemoveFromPanel' />"
-	    "<signal name='Lock' />"
-	    "<signal name='Unlock' />"
 	  "</interface>"
 	"</node>";
 
@@ -2236,22 +2520,12 @@ _panel_applet_setup_x_error_handler (void)
 	_x_error_func = XSetErrorHandler (_x_error_handler);
 }
 
-/**
- * panel_applet_factory_main:
- * @factory_id: Factory ID.
- * @out_process: If the factory is on a separate process or not.
- * @applet_type: GType of the applet this factory creates.
- * @callback: (scope call): Callback to be called when a new applet is to be created.
- * @data: (closure): Callback data.
- *
- * Returns: 0 on success, 1 if there is an error.
- */
-int
-panel_applet_factory_main (const gchar               *factory_id,
-			   gboolean                   out_process,
-			   GType                      applet_type,
-			   PanelAppletFactoryCallback callback,
-			   gpointer                   user_data)
+static int
+_panel_applet_factory_main_internal (const gchar               *factory_id,
+				     gboolean                   out_process,
+				     GType                      applet_type,
+				     PanelAppletFactoryCallback callback,
+				     gpointer                   user_data)
 {
 	PanelAppletFactory *factory;
 	GClosure           *closure;
@@ -2283,6 +2557,84 @@ panel_applet_factory_main (const gchar               *factory_id,
 	return 1;
 }
 
+/**
+ * panel_applet_factory_main:
+ * @factory_id: identifier of an applet factory.
+ * @applet_type: GType of the applet this factory creates.
+ * @callback: (scope call): callback to be called when a new applet is created.
+ * @data: (closure): callback data.
+ *
+ * Creates the applet factory for @factory_id, so that the factory can create
+ * instances of the applet types it is associated with.
+ *
+ * Applet instances created by the applet factory will use @applet_type as
+ * GType. Unless you subclass #PanelApplet, you should use %PANEL_TYPE_APPLET
+ * as @applet_type.
+ *
+ * On creation of the applet instances, @callback is called to setup the
+ * applet. If @callback returns %FALSE, the creation of the applet instance is
+ * cancelled.
+ *
+ * If using C, it is recommended to use #PANEL_APPLET_OUT_PROCESS_FACTORY
+ * instead as it will create a main() function for you.
+ *
+ * It can only be used once, and is incompatible with the use of
+ * %PANEL_APPLET_IN_PROCESS_FACTORY and %PANEL_APPLET_OUT_PROCESS_FACTORY.
+ *
+ * Returns: 0 on success, 1 if there is an error.
+ **/
+int
+panel_applet_factory_main (const gchar               *factory_id,
+			   GType                      applet_type,
+			   PanelAppletFactoryCallback callback,
+			   gpointer                   data)
+{
+	return _panel_applet_factory_main_internal (factory_id, TRUE, applet_type,
+						    callback, data);
+}
+
+/**
+ * panel_applet_factory_setup_in_process: (skip)
+ * @factory_id: identifier of an applet factory.
+ * @applet_type: GType of the applet this factory creates.
+ * @callback: (scope call): callback to be called when a new applet is created.
+ * @data: (closure): callback data.
+ *
+ * Creates the applet factory for @factory_id, so that the factory can create
+ * instances of the applet types it is associated with.
+ *
+ * Applet instances created by the applet factory will use @applet_type as
+ * GType. Unless you subclass #PanelApplet, you should use %PANEL_TYPE_APPLET
+ * as @applet_type.
+ *
+ * On creation of the applet instances, @callback is called to setup the
+ * applet. If @callback returns %FALSE, the creation of the applet instance is
+ * cancelled.
+ *
+ * It can only be used once, and is incompatible with the use of
+ * %PANEL_APPLET_IN_PROCESS_FACTORY and %PANEL_APPLET_OUT_PROCESS_FACTORY.
+ *
+ * Returns: 0 on success, 1 if there is an error.
+ **/
+int
+panel_applet_factory_setup_in_process (const gchar               *factory_id,
+				       GType                      applet_type,
+				       PanelAppletFactoryCallback callback,
+				       gpointer                   data)
+{
+	return _panel_applet_factory_main_internal (factory_id, FALSE, applet_type,
+						    callback, data);
+}
+
+/**
+ * panel_applet_set_background_widget:
+ * @applet: a #PanelApplet.
+ * @widget: a #GtkWidget.
+ *
+ * Configure #PanelApplet to automatically draw the background of the applet on
+ * @widget. It is generally enough to call this function with @applet as
+ * @widget.
+ **/
 void
 panel_applet_set_background_widget (PanelApplet *applet,
 				    GtkWidget   *widget)
